/*
 * Program memory segment support.
 *
 * Copyright (C) 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/err.h>

#include <ipc/mem_ipc.h>
#include <mem/memory_utils.h>

#include <stdio.h>
#include <string.h>

#include "segment.h"



/* Initialise a memory segment. */

Segment::Segment(offset_t base, offset_t size, l4re_rm_flags_t flags,
                 offset_t file_offset, offset_t file_contents)
: base(base), size(size), flags(flags), file_offset(file_offset),
  file_contents(file_contents)
{
  region_base = trunc(base, L4_PAGESIZE);
  region_offset = base - region_base;
  region_size = round(size, L4_PAGESIZE);
}

/* Allocate a region for the segment. */

long Segment::allocate()
{
  return ipc_allocate_align(size, L4RE_RM_F_SEARCH_ADDR | flags,
                            L4_PAGESHIFT, (void **) &buf, &ds);
}

/* Fill a segment region with file content. */

long Segment::fill(file_t *file)
{
  if (!file_contents)
    return L4_EOK;

  memset(buf, 0, region_size);
  client_seek(file, file_offset, SEEK_SET);
  offset_t nread = client_read(file, buf + region_offset, file_contents);

  if (nread < file_contents)
    return -L4_EIO;
  else
    return L4_EOK;
}

/* Define and return the mapped region for the segment. */

MappedRegion &Segment::region()
{
  _region = MappedRegion((l4_addr_t) buf, page_order(region_size), flags, region_base);
  return _region;
}

/* Return the mapped address corresponding to the given address. */

l4_addr_t Segment::region_address(char *address)
{
  return (l4_addr_t) ((address - buf) + (char *) region_base);
}

l4_addr_t Segment::region_address(l4_addr_t address)
{
  return (address - (l4_addr_t) buf) + region_base;
}

/* vim: tabstop=2 expandtab shiftwidth=2
*/
